"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getMain = getMain;
exports.getPackageEntryPoints = getPackageEntryPoints;

var path = _interopRequireWildcard(require("path"));

var fse = _interopRequireWildcard(require("fs-extra"));

var _getModularRoot = _interopRequireDefault(require("../../utils/getModularRoot"));

var _getPackageMetadata = _interopRequireDefault(require("../../utils/getPackageMetadata"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function getMain(packagePath, includePrivate, packageJson) {
  let main;

  if (!packageJson) {
    throw new Error(`no package.json in ${packagePath}, bailing...`);
  }

  if (!packageJson.name) {
    throw new Error(`package.json does not have a valid "name", bailing...`);
  }

  if (!packageJson.version) {
    throw new Error(`package.json does not have a valid "version", bailing...`);
  }

  if (packageJson.module) {
    throw new Error(`package.json shouldn't have a "module" field, bailing...`);
  }

  if (packageJson.typings) {
    throw new Error(`package.json shouldn't have a "typings" field, bailing...`);
  }

  if (!includePrivate && packageJson.private === true) {
    throw new Error(`${packagePath} is marked private, bailing...`);
  }

  if (packageJson.main) {
    main = packageJson.main;
  } else {
    throw new Error(`package.json at ${packagePath} does not have a "main" field, bailing...`);
  }

  if (!fse.existsSync(path.join((0, _getModularRoot.default)(), packagePath, main))) {
    throw new Error(`package.json does not have a main file that points to an existing source file, bailing...`);
  }

  return main;
}

async function getPackageEntryPoints(packagePath, includePrivate) {
  const {
    packageJsonsByPackagePath
  } = await (0, _getPackageMetadata.default)();
  const packageJson = packageJsonsByPackagePath[packagePath];
  return getMain(packagePath, includePrivate, packageJson);
}
//# sourceMappingURL=getPackageEntryPoints.js.map