"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var fs = _interopRequireWildcard(require("fs-extra"));

var _module = require("module");

var svgr = _interopRequireWildcard(require("@svgr/core"));

var _path = _interopRequireDefault(require("path"));

var _getModularRoot = _interopRequireDefault(require("../../utils/getModularRoot"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const svgrOptions = {
  template(variables, {
    tpl
  }) {
    return tpl`
  ${variables.imports};
  
  ${variables.interfaces};
  
  export function ${variables.componentName}(${variables.props}) {
    return (
      ${variables.jsx}
    );
  }
  `;
  }

};

function createPlugin() {
  const plugin = {
    name: 'svgr',

    setup(build) {
      const modularRoot = (0, _getModularRoot.default)();
      const fileContent = new Map();

      function readModularContent(pathName) {
        if (fileContent.has(pathName)) {
          return fileContent.get(pathName);
        } else {
          const joinedPath = _path.default.join(modularRoot, pathName);

          const content = fs.readFileSync(joinedPath, 'utf8');
          fileContent.set(pathName, content);
          return Promise.resolve(content);
        }
      }

      build.onResolve({
        filter: /@svgurl:.*/
      }, args => {
        return {
          pluginData: { ...args
          },
          path: args.path.slice('@svgurl:'.length),
          namespace: 'svgurl'
        };
      });
      build.onLoad({
        filter: /.*/,
        namespace: 'svgurl'
      }, async args => {
        const {
          resolveDir
        } = args.pluginData;
        const contents = await readModularContent(args.path);
        return {
          resolveDir,
          contents,
          loader: 'file'
        };
      });
      build.onResolve({
        filter: /\.svg$/,
        namespace: 'file'
      }, args => {
        const resolver = (0, _module.createRequire)(args.importer);
        const resolvedPathName = resolver.resolve(args.path);
        return {
          pluginData: { ...args
          },
          path: resolvedPathName
        };
      });
      build.onLoad({
        filter: /\.svg$/,
        namespace: 'file'
      }, async args => {
        const pluginData = args.pluginData;

        const pathName = _path.default.relative(modularRoot, args.path);

        const contents = await readModularContent(pathName);

        if (pluginData.kind === 'url-token') {
          return {
            resolveDir: pluginData.resolveDir,
            contents,
            loader: 'dataurl'
          };
        } else {
          const transformedContents = await svgr.transform(contents, svgrOptions, {
            componentName: 'ReactComponent'
          });
          return {
            resolveDir: pluginData.resolveDir,
            contents: `
            export { default } from "@svgurl:${pathName}";
            
            ${transformedContents}
            `,
            loader: 'jsx'
          };
        }
      });
    }

  };
  return plugin;
}

var _default = createPlugin;
exports.default = _default;
//# sourceMappingURL=svgr.js.map