"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = start;

var esbuild = _interopRequireWildcard(require("esbuild"));

var _chalk = _interopRequireDefault(require("chalk"));

var express = _interopRequireWildcard(require("express"));

var _expressWs = _interopRequireDefault(require("express-ws"));

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _mime = require("mime");

var _isCi = _interopRequireDefault(require("is-ci"));

var _memoize = _interopRequireDefault(require("../../utils/memoize"));

var _createPaths = _interopRequireDefault(require("../../utils/createPaths"));

var _getClientEnvironment = _interopRequireDefault(require("../config/getClientEnvironment"));

var _incrementalCompile = _interopRequireDefault(require("./plugins/incrementalCompile"));

var _incrementalReporter = _interopRequireDefault(require("./plugins/incrementalReporter"));

var _wsReload = _interopRequireDefault(require("./plugins/wsReload"));

var _metafileReporter = _interopRequireDefault(require("./plugins/metafileReporter"));

var _firstCompile = _interopRequireDefault(require("./plugins/firstCompile"));

var _openBrowser = _interopRequireDefault(require("../utils/openBrowser"));

var logger = _interopRequireWildcard(require("../../utils/logger"));

var _urls = _interopRequireDefault(require("../config/urls"));

var _api = require("../api");

var _createEsbuildConfig = _interopRequireDefault(require("../config/createEsbuildConfig"));

var _errorOverlayMiddleware = _interopRequireDefault(require("../../../react-dev-utils/errorOverlayMiddleware.js"));

var _getHost = _interopRequireDefault(require("./utils/getHost"));

var _getPort = _interopRequireDefault(require("./utils/getPort"));

var _sanitizeMetafile = _interopRequireWildcard(require("../utils/sanitizeMetafile"));

var _getModularRoot = _interopRequireDefault(require("../../utils/getModularRoot"));

var _rewriteDependenciesPlugin = require("../plugins/rewriteDependenciesPlugin");

var _createEsbuildBrowserslistTarget = _interopRequireDefault(require("../../utils/createEsbuildBrowserslistTarget"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const RUNTIME_DIR = path.join(__dirname, 'runtime');

class DevServer {
  watching = false;

  constructor(paths, urls, host, port, isApp, dependencies, resolutions) {
    this.paths = paths;
    this.urls = urls;
    this.host = host;
    this.port = port;
    this.isApp = isApp;
    this.dependencies = dependencies;
    this.resolutions = resolutions;
    this.firstCompilePromise = new Promise(resolve => {
      this.firstCompilePromiseResolve = resolve;
    });
    this.env = (0, _getClientEnvironment.default)(paths.publicUrlOrPath.slice(0, -1));
    this.express = express.default();
    this.ws = (0, _expressWs.default)(this.express);
    this.express.use(this.handleStaticAsset);
    this.isApp || this.express.get('/static/js/_trampoline.js', this.handleTrampoline);
    this.express.use('/static/js', this.handleStaticAsset);
    this.express.use(this.handleRuntimeAsset);
    this.express.use(express.static(paths.appPublic, {
      cacheControl: false,
      index: false
    })); // eslint-disable-next-line @typescript-eslint/no-misused-promises

    this.express.get('/', this.handleIndex);
    this.express.use((0, _errorOverlayMiddleware.default)());
    this.ws.app.ws('/_ws', (ws, req) => {
      logger.debug('Connected');
    }); // This registers user provided middleware for proxy reasons

    if (fs.existsSync(paths.proxySetup)) {
      // eslint-disable-next-line @typescript-eslint/no-unsafe-call, @typescript-eslint/no-var-requires
      require(paths.proxySetup)(this.express);
    }
  }

  async start() {
    // force clearing the terminal when we start a dev server process
    // unless we're in CI because we'll want to keep all logs
    logger.clear();
    logger.log(_chalk.default.cyan('Starting the development server...\n')); // Start the esbuild before we startup the server

    await this.startEsbuildServer();
    await this.hostRuntime();
    return new Promise((resolve, reject) => {
      try {
        // eslint-disable-next-line @typescript-eslint/no-misused-promises
        this.server = this.express.listen(this.port, this.host, async () => {
          await (0, _openBrowser.default)(this.urls.localUrlForBrowser);
          resolve(this);
        });
      } catch (err) {
        logger.error(err);
        reject(err);
      }
    });
  }

  shutdown = () => {
    var _this$esbuild, _this$esbuild$stop, _this$server;

    (_this$esbuild = this.esbuild) === null || _this$esbuild === void 0 ? void 0 : (_this$esbuild$stop = _this$esbuild.stop) === null || _this$esbuild$stop === void 0 ? void 0 : _this$esbuild$stop.call(_this$esbuild);
    this.ws.getWss().close();
    (_this$server = this.server) === null || _this$server === void 0 ? void 0 : _this$server.close();
    process.nextTick(() => {
      this.ws.getWss().clients.forEach(socket => {
        socket.terminate();
      });
    });
  };
  hostRuntime = (0, _memoize.default)(async () => {
    const runtimeDirFiles = await fs.readdir(RUNTIME_DIR);
    const indexFiles = runtimeDirFiles.filter(p => p.startsWith('index') && ['.js', '.ts'].some(extension => p.endsWith(extension)));

    if (indexFiles.length !== 1) {
      throw new Error(`Found multiple possible entry files`);
    }

    const entryPoint = indexFiles[0];
    this.runtimeEsbuild = await esbuild.build({
      entryPoints: [path.join(RUNTIME_DIR, entryPoint)],
      bundle: true,
      format: 'esm',
      target: 'es2015',
      logLevel: 'silent',
      define: {
        global: 'window'
      },
      banner: {
        js: `window.process = {
          platform: '${process.platform}',
          env: { NODE_ENV: 'developement' },
        }`
      },
      write: false,
      outbase: RUNTIME_DIR,
      absWorkingDir: (0, _getModularRoot.default)(),
      outdir: path.join(RUNTIME_DIR, '_runtime')
    });
  });
  baseEsbuildConfig = (0, _memoize.default)(() => {
    const browserTarget = (0, _createEsbuildBrowserslistTarget.default)(this.paths.appPath);
    return (0, _createEsbuildConfig.default)(this.paths, {
      write: false,
      minify: false,
      entryNames: 'static/js/[name]',
      chunkNames: 'static/js/[name]',
      assetNames: 'static/media/[name]',
      target: browserTarget,
      plugins: this.isApp ? undefined : [(0, _rewriteDependenciesPlugin.createRewriteDependenciesPlugin)(this.dependencies, this.resolutions, browserTarget)]
    });
  });
  runEsbuild = async () => {
    return esbuild.build(this.baseEsbuildConfig());
  };
  metafileCallback = metafile => {
    if (metafile) {
      this.metafile = (0, _sanitizeMetafile.default)(this.paths, metafile);
    }
  };
  firstCompilePluginCallback = () => {
    var _this$firstCompilePro;

    (_this$firstCompilePro = this.firstCompilePromiseResolve) === null || _this$firstCompilePro === void 0 ? void 0 : _this$firstCompilePro.call(this);
  };
  watchEsbuild = async () => {
    var _config$plugins, _config$plugins2, _config$plugins3, _config$plugins4, _config$plugins5;

    const config = this.baseEsbuildConfig();
    (_config$plugins = config.plugins) === null || _config$plugins === void 0 ? void 0 : _config$plugins.push((0, _incrementalReporter.default)(this.paths));
    (_config$plugins2 = config.plugins) === null || _config$plugins2 === void 0 ? void 0 : _config$plugins2.push((0, _incrementalCompile.default)(this.paths, this.urls));
    (_config$plugins3 = config.plugins) === null || _config$plugins3 === void 0 ? void 0 : _config$plugins3.push((0, _wsReload.default)('app', this.ws.getWss(), this.paths));
    (_config$plugins4 = config.plugins) === null || _config$plugins4 === void 0 ? void 0 : _config$plugins4.push((0, _metafileReporter.default)(this.metafileCallback));
    (_config$plugins5 = config.plugins) === null || _config$plugins5 === void 0 ? void 0 : _config$plugins5.push((0, _firstCompile.default)(this.firstCompilePluginCallback));
    this.esbuild = await esbuild.build({ ...config,
      incremental: true,
      watch: {
        onRebuild: (_, result) => {
          if (result) {
            this.esbuild = result;
          }
        }
      }
    });
  };
  startEsbuildServer = async () => {
    if (!this.watching) {
      this.watching = true;
    }

    await this.runEsbuild(); // if the non-incremental succeeds then
    // we start a watching server

    await this.watchEsbuild();
    return this.firstCompilePromise;
  };
  handleIndex = async (_, res) => {
    // wait until the first watch compile is complete
    await this.firstCompilePromise;
    res.writeHead(200);

    if (this.isApp) {
      res.end(await (0, _api.createIndex)({
        paths: this.paths,
        metafile: this.metafile,
        replacements: this.env.raw,
        includeRuntime: true
      }));
    } else {
      res.end(await (0, _api.createIndex)({
        paths: this.paths,
        metafile: this.metafile,
        replacements: this.env.raw,
        includeRuntime: true,
        indexContent: _api.indexFile,
        includeTrampoline: true
      }));
    }
  };
  handleTrampoline = async (_, res) => {
    res.setHeader('content-type', 'application/javascript');
    res.writeHead(200);
    const baseConfig = this.baseEsbuildConfig();
    const trampolineBuildResult = await (0, _api.createViewTrampoline)('index.js', this.paths.appSrc, this.dependencies, this.resolutions, baseConfig.target);
    res.end(trampolineBuildResult.outputFiles[0].text);
  };
  serveEsbuild = (outputDirectory, url, result, res, next) => {
    const outputFiles = result.outputFiles || [];

    for (const file of outputFiles) {
      if ((0, _sanitizeMetafile.sanitizeFileName)('/' + path.relative(outputDirectory, file.path)) === url) {
        const type = (0, _mime.getType)(url);
        res.setHeader('Content-Type', type);
        res.writeHead(200);
        return res.end(file.text);
      }
    }

    next();
  };
  handleStaticAsset = async (req, res, next) => {
    // wait until the first watch compile is complete
    await this.firstCompilePromise;
    this.serveEsbuild(this.baseEsbuildConfig().outdir, req.url, this.esbuild, res, next);
  };
  handleRuntimeAsset = (req, res, next) => {
    this.serveEsbuild(RUNTIME_DIR, req.url, this.runtimeEsbuild, res, next);
  };
}

async function start(target, isApp, packageDependencies, packageResolutions) {
  const paths = await (0, _createPaths.default)(target);
  const host = (0, _getHost.default)();
  const port = await (0, _getPort.default)(host);
  const urls = (0, _urls.default)(process.env.HTTPS === 'true' ? 'https' : 'http', host, port, paths.publicUrlOrPath.slice(0, -1));
  const devServer = new DevServer(paths, urls, host, port, isApp, packageDependencies, packageResolutions);
  const server = await devServer.start();
  ['SIGINT', 'SIGTERM'].forEach(sig => {
    process.on(sig, () => {
      void server.shutdown();
    });
  });

  if (!_isCi.default) {
    // Gracefully exit when stdin ends
    process.stdin.on('end', () => {
      void server.shutdown();
    });
  }
}
//# sourceMappingURL=index.js.map