"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _changeCase = require("change-case");

var _actionPreflightCheck = _interopRequireDefault(require("./utils/actionPreflightCheck"));

var _isModularType = _interopRequireDefault(require("./utils/isModularType"));

var _execAsync = _interopRequireDefault(require("./utils/execAsync"));

var _getLocation = _interopRequireDefault(require("./utils/getLocation"));

var _stageView = _interopRequireDefault(require("./utils/stageView"));

var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));

var _getWorkspaceInfo = _interopRequireDefault(require("./utils/getWorkspaceInfo"));

var _setupEnv = require("./utils/setupEnv");

var _checkBrowsers = require("./utils/checkBrowsers");

var _checkRequiredFiles = _interopRequireDefault(require("./utils/checkRequiredFiles"));

var _createPaths = _interopRequireDefault(require("./utils/createPaths"));

var logger = _interopRequireWildcard(require("./utils/logger"));

var _createEsbuildBrowserslistTarget = _interopRequireDefault(require("./utils/createEsbuildBrowserslistTarget"));

var _prompts = _interopRequireDefault(require("prompts"));

var _getPackageDependencies = require("./utils/getPackageDependencies");

var _filterDependencies = require("./utils/filterDependencies");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function start(packageName) {
  let target = packageName;
  const workspaceInfo = await (0, _getWorkspaceInfo.default)();

  if (!target) {
    const availablePackages = Object.keys(workspaceInfo);
    const chosenTarget = await (0, _prompts.default)({
      type: 'select',
      name: 'value',
      message: 'Select a package to start',
      choices: availablePackages.map(packageName => ({
        title: packageName,
        value: packageName
      })),
      initial: 0
    });
    target = chosenTarget.value;
  }

  let targetPath = await (0, _getLocation.default)(target);
  await (0, _setupEnv.setupEnvForDirectory)(targetPath);

  if ((0, _isModularType.default)(targetPath, 'package')) {
    throw new Error(`The package at ${targetPath} is not a valid modular app or view.`);
  }

  const isEsmView = (0, _isModularType.default)(targetPath, 'esm-view');
  const isView = (0, _isModularType.default)(targetPath, 'view');

  if (isView) {
    targetPath = (0, _stageView.default)(target);
  } else {
    // in the case we're an app then we need to make sure that users have no incorrectly
    // setup their app folder.
    const paths = await (0, _createPaths.default)(target);
    isEsmView ? await (0, _checkRequiredFiles.default)([paths.appIndexJs]) : await (0, _checkRequiredFiles.default)([paths.appHtml, paths.appIndexJs]);
  }

  await (0, _checkBrowsers.checkBrowsers)(targetPath); // True if there's no preference set - or the preference is for webpack.

  const useWebpack = !process.env.USE_MODULAR_WEBPACK || process.env.USE_MODULAR_WEBPACK === 'true'; // True if the preference IS set and the preference is esbuild.

  const useEsbuild = process.env.USE_MODULAR_ESBUILD && process.env.USE_MODULAR_ESBUILD === 'true';
  const {
    manifest: packageDependencies,
    resolutions: packageResolutions
  } = await (0, _getPackageDependencies.getPackageDependencies)(target);
  const {
    external: externalDependencies,
    bundled: bundledDependencies
  } = (0, _filterDependencies.filterDependencies)({
    dependencies: packageDependencies,
    isApp: !isEsmView,
    workspaceInfo
  });
  const {
    external: externalResolutions,
    bundled: bundledResolutions
  } = (0, _filterDependencies.filterDependencies)({
    dependencies: packageResolutions,
    isApp: !isEsmView,
    workspaceInfo
  }); // If you want to use webpack then we'll always use webpack. But if you've indicated
  // you want esbuild - then we'll switch you to the new fancy world.

  if (!useWebpack || useEsbuild) {
    const {
      default: startEsbuildApp
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('./esbuild-scripts/start')));
    await startEsbuildApp(target, !isEsmView, externalDependencies, externalResolutions);
  } else {
    const startScript = require.resolve('modular-scripts/react-scripts/scripts/start.js');

    const modularRoot = (0, _getModularRoot.default)();
    const targetName = (0, _changeCase.paramCase)(target);
    const browserTarget = (0, _createEsbuildBrowserslistTarget.default)(targetPath);
    logger.debug(`Using target: ${browserTarget.join(', ')}`);
    await (0, _execAsync.default)('node', [startScript], {
      cwd: targetPath,
      log: false,
      // @ts-ignore
      env: {
        ESBUILD_TARGET_FACTORY: JSON.stringify(browserTarget),
        MODULAR_ROOT: modularRoot,
        MODULAR_PACKAGE: target,
        MODULAR_PACKAGE_NAME: targetName,
        MODULAR_IS_APP: JSON.stringify(!isEsmView),
        MODULAR_PACKAGE_DEPS: JSON.stringify({
          externalDependencies,
          bundledDependencies
        }),
        MODULAR_PACKAGE_RESOLUTIONS: JSON.stringify({
          externalResolutions,
          bundledResolutions
        })
      }
    });
  }
}

var _default = (0, _actionPreflightCheck.default)(start);

exports.default = _default;
//# sourceMappingURL=start.js.map