"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isCi = _interopRequireDefault(require("is-ci"));

var _typescript = _interopRequireDefault(require("typescript"));

var _chalk = _interopRequireDefault(require("chalk"));

var _getPackageMetadata = _interopRequireDefault(require("./utils/getPackageMetadata"));

var logger = _interopRequireWildcard(require("./utils/logger"));

var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));

var _actionPreflightCheck = _interopRequireDefault(require("./utils/actionPreflightCheck"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function typecheck() {
  const {
    typescriptConfig
  } = await (0, _getPackageMetadata.default)();
  const {
    _compilerOptions,
    ...rest
  } = typescriptConfig;
  const tsConfig = { ...rest,
    exclude: ['node_modules', 'bower_components', 'jspm_packages', 'tmp', '**/dist-types', '**/dist-cjs', '**/dist-es', 'dist'],
    compilerOptions: {
      noEmit: true
    }
  };
  const diagnosticHost = {
    getCurrentDirectory: () => (0, _getModularRoot.default)(),
    getNewLine: () => _typescript.default.sys.newLine,
    getCanonicalFileName: file => _typescript.default.sys.useCaseSensitiveFileNames ? file : toFileNameLowerCase(file)
  }; // Parse all config except for compilerOptions

  const configParseResult = _typescript.default.parseJsonConfigFileContent(tsConfig, _typescript.default.sys, (0, _getModularRoot.default)());

  if (configParseResult.errors.length > 0) {
    logger.error('Failed to parse your tsconfig.json');
    throw new Error(_typescript.default.formatDiagnostics(configParseResult.errors, diagnosticHost));
  }

  const program = _typescript.default.createProgram(configParseResult.fileNames, configParseResult.options); // Pulled from typescript's getCanonicalFileName logic
  // eslint-disable-next-line  no-useless-escape


  const fileNameLowerCaseRegExp = /[^\u0130\u0131\u00DFa-z0-9\\/:\-_\. ]+/g;

  function toFileNameLowerCase(x) {
    return fileNameLowerCaseRegExp.test(x) ? x.replace(fileNameLowerCaseRegExp, x.toLowerCase()) : x;
  }

  program.getSourceFiles().map(f => f.fileName).sort().forEach(f => {
    logger.debug(f);
  }); // Does not emit files or typings but will add declaration diagnostics to our errors
  // This will ensure that makeTypings will be successful in CI before actually attempting to build

  const emitResult = program.emit();

  const diagnostics = _typescript.default.getPreEmitDiagnostics(program).concat(emitResult.diagnostics);

  if (diagnostics.length) {
    if (_isCi.default) {
      // formatDiagnostics will return a readable list of error messages, each with its own line
      throw new Error(_typescript.default.formatDiagnostics(diagnostics, diagnosticHost));
    } // formatDiagnosticsWithColorAndContext will return a list of errors, each with its own line
    // and provide an expanded snapshot of the line with the error


    throw new Error(_typescript.default.formatDiagnosticsWithColorAndContext(diagnostics, diagnosticHost));
  } // "✓ Typecheck passed"


  logger.log(_chalk.default.green('\u2713 Typecheck passed'));
}

var _default = (0, _actionPreflightCheck.default)(typecheck);

exports.default = _default;
//# sourceMappingURL=typecheck.js.map