"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getTargets;

var _browserslist = _interopRequireDefault(require("browserslist"));

var logger = _interopRequireWildcard(require("./logger"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/** 123 or 123.456 or 123.456.789 */
const VersionSchema = /^(\d+\.\d+\.\d+|\d+\.\d+|\d+)$/; // Taken from https://github.com/ben-eb/caniuse-lite/blob/v1.0.30001218/data/browsers.js

/** https://github.com/evanw/esbuild/blob/v0.11.15/internal/compat/js_table.go#L17-L35 */

const FAMILY_MAPPING = {
  chrome: 'chrome',
  edge: 'edge',
  es: 'es',
  firefox: 'firefox',
  ios_saf: 'ios',
  node: 'node',
  android: 'chrome',
  and_chr: 'chrome',
  and_ff: 'firefox',
  safari: 'safari'
};

function getTargets(dirName) {
  const config = (0, _browserslist.default)(_browserslist.default.loadConfig({
    path: dirName
  }));

  if (!config) {
    throw new Error(`Could not find config in ${dirName}`);
  }

  const failedBrowserTargets = new Set();
  const targets = config.map(entry => {
    var _rawVersionOrRange$re;

    const [rawBrowser, rawVersionOrRange] = entry.split(' ');
    const browserResult = FAMILY_MAPPING[rawBrowser];

    if (!browserResult) {
      if (failedBrowserTargets.has(rawBrowser)) {
        logger.debug(`Could not find esbuild equivalent for ${rawBrowser}`);
      }

      failedBrowserTargets.add(rawBrowser);
      return undefined;
    }

    const rawVersionNormalized = rawVersionOrRange // e.g. 13.4-13.7, take the lower range
    === null || rawVersionOrRange // e.g. 13.4-13.7, take the lower range
    === void 0 ? void 0 : (_rawVersionOrRange$re = rawVersionOrRange // e.g. 13.4-13.7, take the lower range
    .replace(/-[\d.]+$/, '') // all => replace with 1
    ) === null || _rawVersionOrRange$re === void 0 ? void 0 : _rawVersionOrRange$re.replace('all', '1');
    const versionResult = VersionSchema.exec(rawVersionNormalized);

    if (!versionResult) {
      return undefined;
    }

    return {
      target: browserResult,
      version: rawVersionNormalized
    };
  }).filter(x => x != null);

  if (targets.length === 0) {
    throw new Error('Could not resolve any esbuild targets');
  }

  return targets.map(({
    target,
    version
  }) => `${target}${version}`);
}
//# sourceMappingURL=createEsbuildBrowserslistTarget.js.map